%CMSAMPLE   Convert a list of polynome segments into a sampled image
%   image = CMSAMPLE(segments,region,zoomfactor)
%
%   region = [left_edge,right_edge]
%   zoomfactor = 1 by default.

% (C) Copyright 2002-2003               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Cris Luengo, August 2002.

function out = cmsample(in,region,zoomfactor)

% Test
if ~isstruct(in) | ~isfield(in,'coefficients') | ~isfield(in,'offsets')
   error('Expected a segment image.')
end
coef = in.coefficients;
offset = in.offsets;
N = size(coef,2);
if N~=length(offset)-1
   error('Segment image is not correct.')
end
if nargin < 3
   zoomfactor = 1;
else
   if ~isnumeric(zoomfactor) | prod(size(zoomfactor))~=1
      error('ZOOMFACTOR must be a scalar.')
   end
   if zoomfactor<1 | mod(zoomfactor,1)
      error('ZOOMFACTOR must be a positive integer.')
   end
end
if nargin < 2 | isempty(region)
   region = offset([1,end]);
else
   if ~isnumeric(region) | prod(size(region))~=2
      error('REGION must have 2 elements.')
   end
   if region(1) < offset(1) | region(2) > offset(end)
      error('REGION falls (partially) outside of image.')
   end
end

% Sample
out = newim(floor((region(2)-region(1))*zoomfactor+1));
I = 1;
for ii=0:size(out)-2
   x = ii/zoomfactor + region(1);
   while offset(I+1) <= x
      I = I+1;
      %if I > N
      %   if ii==size(out)-1
      %      % last pixel exactly on right boundary of last segment
      %      out(end) = cmevaluatepoly(coef(:,end),offset(end)-offset(end-1));
      %      return
      %   end
      %   error('Assertion failed (1)');
      %end
   end
   out(ii) = cmevaluatepoly(coef(:,I),x-offset(I));
end
x = (size(out)-1)/zoomfactor + region(1);
out(end) = cmevaluatepoly(coef(:,end),x-offset(end));
