%CMSEGMENT   Convert a sampled image into a list of polynome segments
%   segments = CMSAMPLE(image)
%
% segments.coefficients -> coeficients of polynomial: a3,a2,a1,a0
% segments.offset       -> position of left end of each polynome
%
% There is one more offset than polynomes. This one gives the rightmost
% x of the image.

% (C) Copyright 2002-2003               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Cris Luengo, August 2002.

function out = cmsegment(in)

% Test
in = squeeze(dip_image(in));
if ~isa(in,'dip_image') | ndims(in)~=1
   error('Expected a 1D input image.')
end

N = size(in)-1;           % number of segments

% Compute second order derivatives at pixel positions
% REF: Numerical Recepies, section 3.3
% REF: Taschenbuch der Mathematik, sub-subsection 19.7.1.1
g = 0.2679491924311227; % approximate. The real sequence g(ii) goes to this bound very quickly.
p = 4-g;
d = newim(N+1,'double');
d(0) = 0;     % boundary condition
din = newim(N+1,'double');
din(1:end-1) = 3*(in(0:end-2)-2*in(1:end-1)+in(2:end));
for ii=1:N-1
   d(ii) = (din(ii)-d(ii-1))/p;
end
d(N) = 0;   % boundary condition
for ii=N-1:-1:0
   d(ii) = d(ii) - g*d(ii+1);
end

% Compute 3rd order polynomial coeficients
coef = zeros(4,N);
coef(1,:) = double(d(1:end)-d(0:end-1))/3;
coef(2,:) = double(d(0:end-1));
coef(3,:) = double(in(1:end)-in(0:end-1)) - double(2*d(0:end-1)+d(1:end))/3;
coef(4,:) = double(in(0:end-1));

% Set left and right values and positions
offset = 0:N;

out = struct('coefficients',coef,'offsets',offset);
